//@author: dottore
//@help: constant shader to write in texture 3d target
//@tags: GEA
//@credits: 

RWTexture3D<float4> OutputTexture : BACKBUFFER;

float3 res : TARGETSIZE;

Texture2D texture2d <string uiname="Texture";>;
SamplerState g_samLinear <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : VIEWPROJECTION;
};


cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD ;
	float4x4 tWVP : WORLDVIEWPROJECTION ;
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
	float4x4 tColor <string uiname="Color Transform";>;
};

// =============================================================================
// VERTEX SHADER ===============================================================
// =============================================================================

struct VS_IN
{
	float4 posO : POSITION;
	float4 texCd : TEXCOORD0;

};

struct VS_OUT
{
    float4 posW: POSITION;
    float4 texCd: TEXCOORD0;
};

VS_OUT VS(VS_IN In)
{
    VS_OUT Out = (VS_OUT)0;
	
    Out.posW  = mul(In.posO, tWVP);
    Out.texCd = mul(In.texCd, tTex);
	
    return Out;
}

// =============================================================================
// GEOMETRY SHADER =============================================================
// =============================================================================

struct GS_OUT_LAYER
{
	float4 posWVP   	: SV_POSITION;
	float4 texCd	: TEXCOORD0;
	uint   layer 	: SV_RenderTargetArrayIndex;
};

[maxvertexcount(3)]
void GS_SetLayer(triangle VS_OUT In[3],
		inout TriangleStream<GS_OUT_LAYER> triOutputStream)
{
	GS_OUT_LAYER Out;
	
	for(int i=0; i<3 ; ++i)
	{
		if(In[i].posW.z>-1 && In[i].posW.z<1)
		{
			Out.posWVP  = In[i].posW;
			Out.texCd  = In[i].texCd;
			Out.layer = clamp((In[i].posW.z*res.z *0.5) + res.z *0.5, 0, res.z-1); 
			
			triOutputStream.Append(Out);
		}
	}
}

// =============================================================================
// PIXEL SHADER ================================================================
// =============================================================================

float4 PS(GS_OUT_LAYER In): SV_Target
{
    float4 col = texture2d.Sample(g_samLinear,In.texCd.xy) * cAmb;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

// =============================================================================
// TECHNIQUES ==================================================================
// =============================================================================

technique11 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_SetLayer() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}




