
RWTexture3D<float2> OutputTexture : BACKBUFFER;

// ============================================================================
// ============================================================================
// ============================================================================

float time;

// NOISE:
#include "MattNoise.fxh"
float noiseGamma = 1;
float3 noisePosOffset;
float baseFreq = 1;

// SPHERE:
float3 spherePos;
float sphereRadius = 2;
float sphereRadiusGradient = 0.5;
float sphereGamma = 1;
float sphereNoiseAmount;
bool sphereInverse;

// PLANE:
float4x4 tPlane;
float planeDistanceGradient = 3;
float planeGamma = 1;
float planeNoiseAmount;
bool planeInverse;

float minSize;


// ============================================================================
// ============================================================================
// ============================================================================
// ============================================================================
// CLASSES

//Interfaces and Classes definitions
interface iNoiseType
{	float4 Noise(float4 pos); 	};

class cFBM : iNoiseType
{
   float4 Noise(float4 pos)
	{	return fBm(pos)+0.5;	}
};

class cTurbulence : iNoiseType
{
   float4 Noise(float4 pos)
   {	return turbulence(pos);	}
};

class cRidge : iNoiseType
{
   float4 Noise(float4 pos)
   {	return ridgedmf(pos);	}
};

cFBM Perlin;
cTurbulence Turbulence;
cRidge Ridge;

iNoiseType NoiseType <string linkclass="Perlin,Turbulence,Ridge";>;

// ============================================================================
// ============================================================================
// ============================================================================

float3 invsize : INVTARGETSIZE;

[numthreads(16, 8, 4)]
void CSNoise(uint3 i : SV_DispatchThreadID)
{
	float3 p = i;
	p *= invsize;
	
	// NOISE
	float noise = saturate(NoiseType.Noise(float4(p*baseFreq+noisePosOffset,time)));
	noise = pow(noise, noiseGamma)-0.5;
	
	// PLANE
	float4 posPlane = mul(float4(p,1), tPlane);
	float planeGradient = pow(saturate((posPlane.y - noise*planeNoiseAmount) / planeDistanceGradient), planeGamma);
	if(planeInverse) planeGradient = 1 - planeGradient;

	// SPHERE
	float sphereGradient = pow(saturate((length(spherePos - p.xyz)- sphereRadius- noise*sphereNoiseAmount)/sphereRadiusGradient), sphereGamma);
	if(sphereInverse) sphereGradient = 1 - sphereGradient;

	// MIX
	float gradient = max(max(sphereGradient, planeGradient), minSize);

	OutputTexture[i] = float2(gradient, noise+0.5);
}

[numthreads(16, 8, 4)]
void CSNoiseInverse(uint3 i : SV_DispatchThreadID)
{
	float3 p = i;
	p *= invsize;
	
	// NOISE
	float noise = saturate(NoiseType.Noise(float4(p*baseFreq+noisePosOffset,time)));
	noise = pow(noise, noiseGamma)-0.5;
	
	// PLANE
	float4 posPlane = mul(float4(p,1), tPlane);
	float planeGradient = pow(saturate((posPlane.y - noise*planeNoiseAmount) / planeDistanceGradient), planeGamma);
	if(planeInverse) planeGradient = 1 - planeGradient;

	// SPHERE
	float sphereGradient = pow(saturate((length(spherePos - p.xyz)- sphereRadius- noise*sphereNoiseAmount)/sphereRadiusGradient), sphereGamma);
	if(sphereInverse) sphereGradient = 1 - sphereGradient;
	
	// MIX
	float gradient = max(max(sphereGradient, planeGradient), minSize);

	OutputTexture[i] = float2(1-gradient, noise+0.5);
}



technique11 Noise
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSNoise() ) );
	}
}

technique11 Noise_Inverse
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSNoiseInverse() ) );
	}
}



