Texture3D tex <string uiname="Texture 3D";>;

SamplerState g_sam
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
	AddressW = Clamp;
};

cbuffer cbPerDraw : register( b0 )
{
	float4x4 tWVP : WORLDVIEWPROJECTION ;
	int3 res = 1;
	float3 invRes = 1;
	float sizeMult = 1;
	bool multSizeR;
	bool multSizeG;
	bool multSizeB;
	bool multSizeA;
};

// =============================================================================
// VERTEX SHADER ===============================================================
// =============================================================================

struct VS_IN
{
	uint ii : SV_InstanceID;
	float4 PosO : POSITION;
};

struct VS_OUT
{
    float4 PosWVP: SV_POSITION ;	
    float4 RGBA: COLOR ;	
};

VS_OUT VS(VS_IN In)
{
    //inititalize all fields of output struct with 0
    VS_OUT Out = (VS_OUT)0;
	
	float3 pO = In.PosO.xyz;
	
	float3 gridPos = float3(	In.ii % res.x,
								floor(In.ii * invRes.x) % res.y,
								floor(In.ii * (invRes.x*invRes.y)));
	gridPos *= invRes;
	
	float3 uv = gridPos + (0.5 * invRes);	
	uv.y = 1-uv.y;
	float4 data = tex.SampleLevel(g_sam, uv, 0);
	
	pO *= sizeMult * invRes;
	
	// size influenced by data:
	if(multSizeR) pO *= saturate(data.x);
	if(multSizeG) pO *= saturate(data.y);
	if(multSizeB) pO *= saturate(data.z);
	if(multSizeA) pO *= saturate(data.w);
	
	// position offset:
	pO += gridPos - 0.5;
	
    Out.PosWVP  = mul(float4(pO,1), tWVP);
    Out.RGBA = data;
	
    return Out;
}

// =============================================================================
// GEOMETRY SHADER =============================================================
// =============================================================================

//float dthr = 0.0f;
/*
[maxvertexcount(3)]
void GSCull(triangle VS_OUT In[3], inout TriangleStream<VS_OUT> gsout)
{
    VS_OUT output;
	
	if (In[0].Dens > dthr)
	{
		gsout.Append(In[0]);
		gsout.Append(In[1]);
		gsout.Append(In[2]);
	}
	
}
*/

// =============================================================================
// PIXEL SHADER ================================================================
// =============================================================================

float4 PS(VS_OUT In): SV_Target
{
    return In.RGBA;
}

// =============================================================================
// TECHNIQUES ==================================================================
// =============================================================================

technique11 render
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
}





