//@author: dottore
//@help: standard constant shader
//@tags: color
//@credits: 

float4x4 tVP : VIEWPROJECTION;
float4x4 tVI : VIEWINVERSE;

float2 iRes : INVTARGETSIZE ;


Texture3D tex <string uiname="Texture 3D";>;
SamplerState g_sam
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
	AddressW = Clamp;
};

cbuffer cbPerDraw : register( b0 )
{
	float4x4 tWVP : WORLDVIEWPROJECTION ;
	float4x4 tW : WORLD ;
	int3 res = 1;
	float3 invRes = 1;
	float sizeMult = 1;
	bool multSizeR;
	bool multSizeG;
	bool multSizeB;
	bool multSizeA;
};

//==============================================================================
// VERTEX SHADER ===============================================================
//==============================================================================

struct VS_IN
{
	uint iv : SV_VertexID;
};

struct VS_OUT
{
    float3 PosW: POSITION ;	
	//float2 TexCd : TEXCOORD0 ;
	float4 Vcol : COLOR ;
	float size : TEXCOORD1;
};

VS_OUT VS(VS_IN In)
{
    //inititalize all fields of output struct with 0
    VS_OUT Out = (VS_OUT)0;
		
	float3 gridPos = float3(	In.iv % res.x,
								floor(In.iv * invRes.x) % res.y,
								floor(In.iv * (invRes.x*invRes.y)));
	gridPos *= invRes;
	
	float3 uv = gridPos + (0.5 * invRes);
	uv.y = 1-uv.y;
	float4 data = tex.SampleLevel(g_sam, uv, 0);
	
	float size = sizeMult * min(min(invRes.x,invRes.y),invRes.z) * 0.5;
	
	// size influenced by data:
	if(multSizeR) size *= saturate(data.x);
	if(multSizeG) size *= saturate(data.y);
	if(multSizeB) size *= saturate(data.z);
	if(multSizeA) size *= saturate(data.w);
		
    Out.PosW  = gridPos-0.5;//mul(float4(gridPos-0.5,1), tW).xyz;
	Out.Vcol = data; 
	Out.size = size ;

	
    return Out;
}

//==============================================================================
// GEOMETRY SHADER =============================================================
//==============================================================================

float3 g_positions[4]:IMMUTABLE =
{
    float3( -1, 1, 0 ),
    float3( 1, 1, 0 ),
    float3( -1, -1, 0 ),
    float3( 1, -1, 0 ),
};
float2 g_texcoords[4]:IMMUTABLE = 
{ 
    float2(0,1), 
    float2(1,1),
    float2(0,0),
    float2(1,0),
};

struct GS_OUT
{
    float4 PosWVP: SV_POSITION ;	
	float2 TexCd : TEXCOORD0 ;
	float4 Vcol : COLOR ;
};


[maxvertexcount(4)]
void GS(point VS_OUT In[1], inout TriangleStream<GS_OUT> SpriteStream)
{
    GS_OUT Out;
    
    // Emit two new triangles
    for(int i=0; i<4; i++)
    {
        float3 position = g_positions[i] * In[0].size;
        position = mul( position, (float3x3)tVI ) + In[0].PosW;
    	float3 norm = mul(float3(0,0,-1),(float3x3)tVI );
        Out.PosWVP = mul( float4(position,1.0), tWVP );
        
        Out.TexCd = g_texcoords[i];	
        Out.Vcol = In[0].Vcol;
    	
        SpriteStream.Append(Out);
    }
    SpriteStream.RestartStrip();
}


[maxvertexcount(4)]
void GS_pixel(point VS_OUT In[1], inout TriangleStream<GS_OUT> SpriteStream)
{
    GS_OUT Out;
    
    float4 screenPos = mul(float4(In[0].PosW,1), tWVP);
	
    // Emit two new triangles
    for(int i=0; i<4; i++)
    {
        float3 position = g_positions[i] * screenPos.w * float3(iRes* 512,1)  * In[0].size;
    	position += screenPos.xyz;
    	float3 norm = mul(float3(0,0,-1),(float3x3)tVI );
    	
        Out.PosWVP = float4(position, screenPos.w);
        Out.TexCd = g_texcoords[i];	
        Out.Vcol = In[0].Vcol;
    	
        SpriteStream.Append(Out);
    }
    SpriteStream.RestartStrip();
}


//==============================================================================
// PIXEL SHADER ================================================================
//==============================================================================

float4 PS_quad(GS_OUT In): SV_Target
{
	return In.Vcol;
}

float4 PS_circle(GS_OUT In): SV_Target
{
	if(length(In.TexCd-.5)>.5){discard;}

	return In.Vcol;
}

float4 PS_pixel(GS_OUT In): SV_Target
{
	return In.Vcol;
}

//==============================================================================
// TECHIQUES ===================================================================
//==============================================================================

technique11 _quad
{
	pass P0
	{
		
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_quad() ) );
	}
}

technique11 _circle
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_circle() ) );
	}
}

technique11 _pixel
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_pixel() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_pixel() ) );
	}
}

technique11 _pixel_circle
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_pixel() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_circle() ) );
	}
}

