#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;

using SlimDX;


using VVVV.DX11;
using FeralTic.DX11.Resources;
using FeralTic.DX11;
using FeralTic.DX11.Geometry;
#endregion usings

namespace VVVV.Nodes
{
	public struct Vertex
    {
        public Vector2 pos, uv;
        public Vertex(Vector2 p, Vector2 u)
        {
            pos = p;
            uv = u;
        }

    }
	
	#region PluginInfo
	[PluginInfo(Name = "Template", Category = "DX11.Geometry", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class DX11_GeometryTemplateNode : IPluginEvaluate, IDX11ResourceHost
	{
		#region fields & pins
		[Input("Points 2D")]
        IDiffSpread<Vector2> FIn;

        [Output("Geometry")]
        protected ISpread<DX11Resource<DX11IndexedGeometry>> geometryOutput;

		private bool invalidate = false;
		#endregion fields & pins

		//called when data for any output pin is requested
 		public void Evaluate(int SpreadMax)
        {

            if (geometryOutput[0] == null)
            {
                geometryOutput[0] = new DX11Resource<DX11IndexedGeometry>();
            }

            if (FIn.IsChanged)
            {
                geometryOutput[0].Dispose();
                geometryOutput[0] = new DX11Resource<DX11IndexedGeometry>();
            }
        }



        public void Destroy(DX11RenderContext context, bool force)
        {
            if (geometryOutput[0] != null)
            {
                geometryOutput[0].Dispose(context);
            }
        }



        public void Update(DX11RenderContext context)
        {
            if (!geometryOutput[0].Contains(context))
            {

                Vertex[] points = new Vertex[6];

                points[0] = new Vertex(new Vector2(0.0000f, 0.0000f) + FIn[0], new Vector2(0.5000f, 0.5000f));

                points[1] = new Vertex(new Vector2(0.0000f, 0.5000f) + FIn[1], new Vector2(0.5000f, 1.0000f));
                points[2] = new Vertex(new Vector2(-0.4755f, 0.1545f) + FIn[2], new Vector2(0.0245f, 0.6545f));
                points[3] = new Vertex(new Vector2(-0.2939f, -0.4045f) + FIn[3], new Vector2(0.2061f, 0.0955f));
                points[4] = new Vertex(new Vector2(0.2939f, -0.4045f) + FIn[4], new Vector2(0.7939f, 0.0955f));
                points[5] = new Vertex(new Vector2(0.4755f, 0.1545f) + FIn[5], new Vector2(0.9755f, 0.6545f));
  


                int[] indecies = new int[15];

                indecies[0] = 1; indecies[1] = 0; indecies[2] = 2;
                indecies[3] = 2; indecies[4] = 0; indecies[5] = 3;

                indecies[6] = 3; indecies[7] = 0; indecies[8] = 4;
                indecies[9] = 4; indecies[10] = 0; indecies[11] = 5;

                indecies[12] = 5; indecies[13] = 0; indecies[14] = 1;
            

                DX11IndexedGeometry ig = DX11IndexedGeometry.CreateFrom<Vertex>(context, points, indecies, Pos2Tex2Vertex.Layout);
                ig.Topology = SlimDX.Direct3D11.PrimitiveTopology.TriangleList;

                geometryOutput[0][context] = ig;
            }
        }
	}
}
