//@author: antokhio
//@help: template for geometry shaders
//@tags: GS, template
//@credits:

//#include "iqFbm.fxh"

float4x4 tW : WORLD;
float4x4 tVP : VIEWPROJECTION;

struct VS_IN
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
	float4 uv : TEXCOORD0;
};

struct VS_OUT
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
	float4 uv : TEXCOORD0;
};

struct GS_OUT
{
	float4 pos : SV_Position;
	float4 normal : NORMAL;

};

VS_OUT VS(VS_IN input)
{
    return input;
}

float scale = 1;
float time;

float4 GetNormal (float3 p0, float3 p1, float3 p2)
{
	float3 fa = p0 - p1;
	float3 fb = p0 - p2;
	float3 pn = cross(fa, fb);
	return float4 (normalize(pn), 1);
}

float4 tVWP (float3 p)
{
	return mul (float4 (p,1), mul (tW, tVP));
}

[maxvertexcount(60)]
void GS(triangle VS_OUT input[3], inout TriangleStream<GS_OUT> gsout)
{
	GS_OUT output;
	
	float3 n = (input[0].normal + input[1].normal + input[2].normal)/3;
	
	float3 p0 = input[0].pos;
	float3 p1 = input[1].pos;
	float3 p2 = input[2].pos;
	
	float3 p3 = (p0 + p1 + p2)/3 + n  * scale;
		   
	
	output.normal = mul(GetNormal(p0,p3,p2), tW);
	
	output.pos = tVWP(p0);
	
	gsout.Append(output);
	
	output.pos = tVWP(p3);
	gsout.Append(output);
	
	output.pos = tVWP(p2);
	gsout.Append(output);
	
	gsout.RestartStrip();
	
	output.normal = mul(GetNormal(p2,p3,p1), tW);
	
	output.pos = tVWP(p2);
	gsout.Append(output);
	
	output.pos = tVWP(p3);
	gsout.Append(output);
	
	output.pos = tVWP(p1);
	gsout.Append(output);
	
	gsout.RestartStrip();
	
	output.normal = mul(GetNormal(p1,p3,p0), tW);
	
	output.pos = tVWP(p1); 
	gsout.Append(output);
	
	output.pos = tVWP(p3);
	gsout.Append(output);
	
	output.pos =tVWP(p0); 
	gsout.Append(output);
	
	gsout.RestartStrip();
	
}

float4 PS(GS_OUT input): SV_Target
{
    return input.normal;
}

technique10 Template
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( CompileShader( gs_4_0, GS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }  
}