//@author: antokhio
//@help: visualize normals on geometry shader
//@tags: normals
//@credits: antokhio

//StructuredBuffer<float4> color <bool color=true;>;

//Texture2D tex0;
SamplerState s0:IMMUTABLE {Filter=MIN_MAG_MIP_LINEAR;AddressU=CLAMP;AddressV=CLAMP;};

float4x4 tW : WORLD;
float4x4 tVP : VIEWPROJECTION;
float4x4 tWIT : WORLDINVERSETRANSPOSE;

float4 cAmb <bool color=true; String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };

struct VS_IN
{
	float4 pos : POSITION;
	float4 normal : NORMAL;
};

struct VS_OUT
{
    float4 pos: POSITION;
    float4 normal : NORMAL;
};

struct GS_OUT
{
	float4 pos  : SV_POSITION;
};

VS_OUT VS(VS_IN input)
{
    
	//VS_OUT output = (VS_OUT)0;
    //output.pos  = mul(input.pos,mul(tW,tVP));
    //output.uv = mul(input.uv, tTex);
    return input;
}
float nLength = 1;
[maxvertexcount(6)]
void GS_VERTEX_NORMAL(triangle VS_OUT input[3], inout LineStream<GS_OUT> lines)
{
	GS_OUT output;
	for (int i = 0 ; i < 3 ; i ++)
	{
		float4 p = mul (input[i].pos, tW);
		float4 n = mul (input[i].normal, tWIT);
		output.pos = mul (p,tVP);
		lines.Append(output);
		output.pos = mul (p + n * nLength, tVP);
		lines.Append(output);
		lines.RestartStrip();
	}
	
}
[maxvertexcount(2)]
void GS_FACE_NORMAL(triangle VS_OUT input[3], inout LineStream<GS_OUT> lines)
{
	GS_OUT output;
	float4 p = mul((input[0].pos + input[1].pos + input[2].pos)/3,tW);
	float4 n = (input[0].normal + input[1].normal + input[2].normal)/3;
	output.pos = mul (p,tVP);
	lines.Append(output);
	output.pos = mul (p + n * nLength, tVP);
	lines.Append(output);
	lines.RestartStrip();
}

float4 PS(GS_OUT input): SV_Target
{
    return cAmb;
}

technique10 VERTEX_NORMALS
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_VERTEX_NORMAL() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}

technique10 FACE_NORMALS
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_FACE_NORMAL() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}

technique10 FACE_AND_VERTEX_NORMALS
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_VERTEX_NORMAL() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
	pass P1
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_FACE_NORMAL() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




