float4x4 tWVP: WORLDVIEWPROJECTION;
float4x4 tVI : VIEWINVERSE;

float lenght = 1;
float noisePosScale = 0.5;
float noiseOffsetScale = 6;

float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
float time;

#include "noiseFunction.fxh"

#define maxExtrSegm 16
#define invMaxExtrSegm (float)1/maxExtrSegm+0.0001

struct vsInBuffer
{
	float4 posO : POSITION ;
    float4 normO: NORMAL ;
	uint iv : SV_VertexID;
};

struct gsIn
{
    float4 pos: POSITION ;
    float3 norm: NORMAL ;
}; 

struct psIn
{
	float4 pos: SV_POSITION;
    float2 uv: TEXCOORD01 ;
};


gsIn VSBuffer(vsInBuffer input)
{
    gsIn output;
    output.pos  = input.posO;
    output.norm  = input.normO.xyz;

    return output;
}

[maxvertexcount(maxExtrSegm)]
void GS_extrude1(point gsIn input[1], inout TriangleStream<psIn> SpriteStream)
{
    psIn output;
    for(int i=0; i<maxExtrSegm; i++)
    {
        float3 position = input[0].pos.xyz + input[0].norm*i*lenght;

    	position += float3(	inoise(float4(position*noisePosScale,time)),
    						inoise(float4(position*noisePosScale,time+354.25)),
    						inoise(float4(position*noisePosScale,time-63.2))) * pow(i * invMaxExtrSegm, 0.5) * noiseOffsetScale ;

    	output.pos = mul( float4(position,1.0), tWVP );
    	
        output.uv = i * (1/(float)maxExtrSegm);
        SpriteStream.Append(output);
    }
    SpriteStream.RestartStrip();
}

[maxvertexcount(maxExtrSegm)]
void GS_extrude2(point gsIn input[1], inout TriangleStream<psIn> SpriteStream)
{
	float extrLenght = inoise(float4(input[0].pos.xyz*0.1,time))+0.25;
	extrLenght = clamp(extrLenght,0,1);
	extrLenght *= maxExtrSegm*0.99;
	
    psIn output;
    for(int i=0; i<extrLenght; i++)
    {
        float3 position = input[0].pos.xyz + input[0].norm*(i+frac(extrLenght))*lenght;

    	position += float3(	inoise(float4(position*noisePosScale,time)),
    						inoise(float4(position*noisePosScale,time+354.25)),
    						inoise(float4(position*noisePosScale,time-63.2))) * pow(i * invMaxExtrSegm, 0.5) * noiseOffsetScale ;

    	
        output.pos = mul( float4(position,1.0), tWVP );
    	
        output.uv = pow(abs(i* invMaxExtrSegm)+frac(extrLenght)*invMaxExtrSegm, .8);
        SpriteStream.Append(output);
    }
    SpriteStream.RestartStrip();
}

[maxvertexcount(maxExtrSegm)]
void GS_extrude3(point gsIn input[1], inout TriangleStream<psIn> SpriteStream)
{
	float extrLenght = inoise(float4(input[0].pos.xyz*0.1,time))+0.25;
	extrLenght = clamp(extrLenght,0,1);
	extrLenght *= maxExtrSegm*0.99;
	
    psIn output;
    for(int i=0; i<extrLenght; i++)
    {
		float apicePhase = saturate(extrLenght - i);

        float3 position = input[0].pos.xyz + input[0].norm*min((i+apicePhase),extrLenght)*lenght;

    	position += float3(	inoise(float4(position*noisePosScale,time)),
    						inoise(float4(position*noisePosScale,time+354.25)),
    						inoise(float4(position*noisePosScale,time-63.2))) * pow(abs(i*invMaxExtrSegm), 0.5) * noiseOffsetScale ;

    	
        output.pos = mul( float4(position,1.0), tWVP );
    	
        output.uv = pow(abs(i* invMaxExtrSegm)+apicePhase*invMaxExtrSegm, .8);
        SpriteStream.Append(output);
    }
    SpriteStream.RestartStrip();
}


float4 PS_Tex(psIn input): SV_Target
{
	return cAmb*float4(input.uv.xxx,1);
}

technique11 ExtrudeConstantLenght
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VSBuffer() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_extrude1() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_Tex() ) );
	}
} 

technique11 ExtrudeDynamicLenght
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VSBuffer() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_extrude2() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_Tex() ) );
	}
} 

technique11 ExtrudeDynamicLenght2
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VSBuffer() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_extrude3() ) );
		SetPixelShader( CompileShader( ps_5_0, PS_Tex() ) );
	}
} 


 
