
int ringLength = 1;
float currentTime;

struct data
{
	float3 pos;
	float3 vel;
	float bornTime;
};
RWStructuredBuffer<data> Output : BACKBUFFER;

StructuredBuffer<uint> perEmitterOffset;
StructuredBuffer<float3> emitterPos;
StructuredBuffer<float3> emitterVel;
StructuredBuffer<uint> indices;

[numthreads(1, 1, 1)]
void headsCS( uint3 DTid : SV_DispatchThreadID )
{
	uint binID = indices[DTid.x];
	
	uint segmID = perEmitterOffset[binID];
	uint id = (binID * ringLength) + segmID;
	
	Output[id].pos = emitterPos[binID];
	Output[id].vel = emitterVel[binID];
	Output[id].bornTime = currentTime;
}

[numthreads(16, 1, 1)]
void resetCS( uint3 DTid : SV_DispatchThreadID )
{
	uint groupID = DTid.x / ringLength;
	uint segmID = DTid.x % ringLength;
		
	uint binID = indices[groupID];
	uint id = (binID * ringLength) + segmID;
		
	Output[id].pos = emitterPos[binID];
	Output[id].vel = emitterVel[binID];
	Output[id].bornTime = currentTime;
}

//==============================================================================
//==============================================================================
//==============================================================================

technique11 setRingBuffer
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, headsCS() ) );
	}
}

technique11 resetBin
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, resetCS() ) );
	}
}
