
int ringLength = 1;

struct posVel
{
	float3 pos;
	float3 vel;
	float bornTime;
};
StructuredBuffer<posVel> Input;
StructuredBuffer<uint> perEmitterOffset;

RWTexture2D<float4> OutputTexture :BACKBUFFER;
RWStructuredBuffer<posVel> OutputBuffer : BACKBUFFER;

// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================

[numthreads(128, 1, 1)]
void WriteTextureCS(uint3 DTid : SV_DispatchThreadID )
{/*
	uint traceID = DTid.x / ringLength;
	uint traceSegmID = DTid.x % ringLength;
	int index = (traceSegmID + perEmitterOffset[traceID]) % ringLength + traceID * ringLength;
		
	OutputTexture[float2(traceSegmID, traceID)] = 
	float4(Input[index].pos,Input[index].bornTime);
*/
	uint traceID = DTid.x / ringLength;
	uint traceSegmID = DTid.x % ringLength;
	int index = (traceSegmID + perEmitterOffset[traceID]) % ringLength + traceID * ringLength;
		
	OutputTexture[float2((traceSegmID-perEmitterOffset[traceID]-1)%ringLength, traceID)] = 
	float4(Input[DTid.x].pos,Input[DTid.x].bornTime);

}

[numthreads(128, 1, 1)]
void WriteRawTextureCS(uint3 DTid : SV_DispatchThreadID )
{
	uint traceID = DTid.x / ringLength;
	uint traceSegmID = DTid.x % ringLength;
		
	OutputTexture[float2(traceSegmID, traceID)] = 
	float4(Input[DTid.x].pos,Input[DTid.x].bornTime);
}

[numthreads(128, 1, 1)]
void WriteBufferCS(	uint3 DTid : SV_DispatchThreadID )
{
	uint traceID = DTid.x / ringLength;
	uint traceSegmID = DTid.x % ringLength+1;
	int index = (traceSegmID + perEmitterOffset[traceID]) % ringLength + traceID * ringLength;
		
	OutputBuffer[DTid.x] = Input[index];
}

// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================
// =============================================================================

technique11 RingBuffer_Reordered_to_texture
{
	pass P0
	{
		SetComputeShader(CompileShader( cs_5_0, WriteTextureCS() ) );
	}
}

technique11 RingBuffer_Raw_to_texture
{
	pass P0
	{
		SetComputeShader(CompileShader( cs_5_0, WriteRawTextureCS() ) );
	}
}

technique11 RingBuffer_Reordered_to_buffer
{
	pass P0
	{
		SetComputeShader(CompileShader( cs_5_0, WriteBufferCS() ) );
	}
}

