//@author: dottore
//@help: append buffer template
//@tags: template
//@credits: 

struct tri
{
	float3 v_1;
	float3 v_1_info;
	float3 v_2;
	float3 v_2_info;
	float3 v_3;
	float3 v_3_info;
};
AppendStructuredBuffer<tri> gOutput : BACKBUFFER;

StructuredBuffer<float3> SeedPosition;
//StructuredBuffer<float3> SeedDirection;
StructuredBuffer<float> extrudeAmount;

#include "iqNoise.fxh"

#define pi 3.14159265358979
#define twoPi 6.283185307179586


// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// EXTRUDE STEPS COUNT DEPENDS ON EXTRUDE AMOUNT INPUT BUFFER

[numthreads(1, 1, 1)]
void appendCS_1( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float extrAmount = extrudeAmount[DTid.x];	
	
	tri Out ;
		for (int i = 0; i < extrAmount; i++)
		{
			float minGrow = max(extrAmount-i, 0);
			float maxGrow = min(extrAmount-i, 1);
			float leafGrow = clamp(extrAmount, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			Out.v_1 = seedPos + float3(0,i,0);
			Out.v_2 = seedPos + float3(0,i+leafGrow,0);
			Out.v_3 = seedPos + float3(leafGrow,i,0);

			Out.v_1_info = float3(extrAmount, leafGrow, i);
			Out.v_2_info = float3(extrAmount, leafGrow, i);
			Out.v_3_info = float3(extrAmount, leafGrow, i);
			gOutput.Append(Out);	
		}
}

// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// DIRECTION IS GIVEN BY GROWING INFO

[numthreads(1, 1, 1)]
void appendCS_2( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float3 prevSeed = seedPos;
	
	float extrAmount = extrudeAmount[DTid.x];	
	
	tri Out ;
		for (int i = 0; i < extrAmount; i++)
		{
			float minGrow = max(extrAmount-i, 0);
			float maxGrow = min(extrAmount-i, 1);
			float leafGrow = clamp(extrAmount, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			// SEED extruded new position:
			float3 prevSeed = seedPos;
			float angle = max((i / extrAmount)*2-1,0) * 0.5+0.25;
			angle *= twoPi;
			seedPos += float3(0, sin(angle),cos(angle)) * leafGrow;

			// FINAL VERTICES POSITION AND DATA
			Out.v_1 = seedPos + float3(-.25*leafGrow,0,0);
			Out.v_2 = prevSeed ;
			Out.v_3 = seedPos + float3(0.25*leafGrow,0,0);

			Out.v_1_info = float3(extrAmount, leafGrow, i);
			Out.v_2_info = float3(extrAmount, leafGrow, i);
			Out.v_3_info = float3(extrAmount, leafGrow, i);
			
			gOutput.Append(Out);	
		}
}

// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// BENDING IS GIVEN BY GROWING INFO

[numthreads(1, 1, 1)]
void appendCS_3( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float3 prevSeed = seedPos;
	
	float extrAmount = extrudeAmount[DTid.x];	
	
	tri Out ;
		for (int i = 0; i < 16; i++)
		{
			float minGrow = max(extrAmount-i, 0);
			float maxGrow = min(extrAmount-i, 1);
			float leafGrow = clamp(extrAmount, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			float bend = branchGrow;
			// SEED extruded new position:
			float3 prevSeed = seedPos;
			float angle = branchGrow *0.05+0.25;
			angle *= twoPi;
			seedPos += float3(0, sin(angle),cos(angle)) ;

			// FINAL VERTICES POSITION AND DATA
			Out.v_1 = seedPos + float3(-.25,0,0);
			Out.v_2 = prevSeed ;
			Out.v_3 = seedPos + float3(0.25,0,0);

			Out.v_1_info = float3(extrAmount, leafGrow, i);
			Out.v_2_info = float3(extrAmount, leafGrow, i);
			Out.v_3_info = float3(extrAmount, leafGrow, i);
			
			gOutput.Append(Out);	
		}
}

// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// BENDING IS GIVEN BY GROWING INFO

[numthreads(1, 1, 1)]
void appendCS_4( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float3 prevSeed = seedPos;
	
	float extrAmount = extrudeAmount[DTid.x];	
	
	tri Out ;
		for (int i = 0; i < 16; i++)
		{
			float minGrow = max(extrAmount-i, 0);
			float maxGrow = min(extrAmount-i, 1);
			float leafGrow = clamp(extrAmount, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			float bend = branchGrow;
			// SEED extruded new position:
			float3 prevSeed = seedPos;
			float angle = extrAmount * i*i *0.0005+0.25;
			angle *= twoPi;
			seedPos += float3(0, sin(angle),cos(angle)) ;

			// FINAL VERTICES POSITION AND DATA
			Out.v_1 = seedPos + float3(-.25,0,0);
			Out.v_2 = prevSeed ;
			Out.v_3 = seedPos + float3(0.25,0,0);

			Out.v_1_info = float3(abs(extrAmount), leafGrow, i);
			Out.v_2_info = Out.v_3_info = Out.v_1_info;
			
			gOutput.Append(Out);	
		}
}

// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// BENDING IS GIVEN BY GROWING INFO

[numthreads(1, 1, 1)]
void appendCS_5( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float3 prevSeed = seedPos;
	
	float extrAmount = extrudeAmount[DTid.x];	
	float extr = abs(extrAmount);
	
	tri Out ;
		for (int i = 0; i < extr; i++)
		{
			float minGrow = max(extr-i, 0);
			float maxGrow = min(extr-i, 1);
			float leafGrow = clamp(extr, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			float bend = branchGrow;
			// SEED extruded new position:
			float3 prevSeed = seedPos;
			float angle = extrAmount * i*i *0.0005+0.25;
			angle *= twoPi;
			seedPos += float3(0, sin(angle),cos(angle)) *leafGrow;

			// FINAL VERTICES 1 POSITION AND DATA
			Out.v_1 = seedPos + float3(-.25,0,0);
			Out.v_2 = prevSeed + float3(-.25,0,0);
			Out.v_3 = seedPos + float3(0.25,0,0);

			Out.v_1_info = float3(abs(extrAmount), leafGrow, i);
			Out.v_2_info = Out.v_3_info = Out.v_1_info;
			
			gOutput.Append(Out);
		
			// FINAL VERTICES 2 POSITION AND DATA
			Out.v_1 = prevSeed + float3(-.25,0,0);
			Out.v_3 = seedPos + float3(.25,0,0);
			Out.v_2 = prevSeed + float3(0.25,0,0);

			Out.v_1_info = float3(abs(extrAmount), leafGrow, i);
			Out.v_2_info = Out.v_3_info = Out.v_1_info;
			
			gOutput.Append(Out);

			
		}
}


// =============================================================================
// SIMPLE TRIANGLE CHAIN GROW FROM POINT.
// BENDING IS GIVEN BY GROWING INFO

[numthreads(1, 1, 1)]
void appendCS_6( uint3 DTid : SV_DispatchThreadID )
{	

	// growing info:
	float3 seedPos = SeedPosition[DTid.x];
	float3 prevSeed = seedPos;
	
	float extrAmount = extrudeAmount[DTid.x];	
	
	tri Out ;
		for (int i = 0; i < extrAmount; i++)
		{
			float minGrow = max(extrAmount-i, 0);
			float maxGrow = min(extrAmount-i, 1);
			float leafGrow = clamp(extrAmount, minGrow, maxGrow);
			float branchGrow = leafGrow + i;	
			
			float bend = branchGrow;
			// SEED extruded new position:
			float3 prevSeed = seedPos;
			float angle = extrAmount * i*i *0.0005+0.25;
			angle *= twoPi;
			seedPos += float3(	iqNoise3f(seedPos*4), 
								iqNoise3f(seedPos*4+5.32) +0.5, 
								iqNoise3f(seedPos*4-51.3)) *leafGrow ;

			// FINAL VERTICES POSITION AND DATA
			Out.v_1 = seedPos;
			Out.v_2 = prevSeed ;
			Out.v_3 = seedPos + float3(0.25,0,0);

			Out.v_1_info = float3(abs(extrAmount), leafGrow, i);
			Out.v_2_info = Out.v_3_info = Out.v_1_info;
			
			gOutput.Append(Out);	
		}
}

//==============================================================================
//==============================================================================
//==============================================================================

technique11 emit_triangles
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_1() ) );
	}
}

technique11 emit_triangles_direction
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_2() ) );
	}
}


technique11 emit_triangles_bend
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_3() ) );
	}
}

technique11 emit_triangles_bend2
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_4() ) );
	}
}

technique11 emit_triangles_bend3_stripes
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_5() ) );
	}
}

technique11 emit_triangles_flower
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, appendCS_6() ) );
	}
}
