//@author: dottore
//@help: streamout template
//@tags: template
//@credits: 

//==============================================================================
//PARAMETERS ===================================================================
//==============================================================================

float4x4 tW ;
float4x4 tWVP : WORLDVIEWPROJECTION ;

struct tri
{
	float3 v;
	float3 v_info;
};
StructuredBuffer<tri> trianglesBuffer ;

struct GS_IN
{
    uint index: TEXCOORD ;
}; 

struct GS_STREAMOUT
{
    float4 posW : SV_POSITION ;
    float3 normW : NORMAL ;
	float4 texCd : TEXCOORD1 ;
};

//==============================================================================
// VERTEX SHADER: ==============================================================
//==============================================================================

GS_IN VS(uint iv : SV_VertexID)
{
    GS_IN Out;
	
    Out.index  = iv * 3;

    return Out;
}

//==============================================================================
// GEOMETRY SHADER: ============================================================
//==============================================================================

[maxvertexcount(3)]
void GS_1( point GS_IN In[1], inout TriangleStream<GS_STREAMOUT> OutputStream )
{	
    GS_STREAMOUT Out = (GS_STREAMOUT)0;

	uint id = In[0].index;
    for( uint i=0; i<3; i++ )
    {
        Out.posW = mul(float4(trianglesBuffer[id+i].v,1), tWVP);
        Out.normW = 1;
		Out.texCd = float4(trianglesBuffer[id+i].v_info,1);
    	
        OutputStream.Append( Out );
    }
    OutputStream.RestartStrip();
}

// NORMAL FACE

[maxvertexcount(3)]
void GS_2( triangle GS_IN In[3], inout TriangleStream<GS_STREAMOUT> OutputStream )
{	
    GS_STREAMOUT Out = (GS_STREAMOUT)0;

	uint id = In[0].index;

	float3 pos_1 = trianglesBuffer[id].v;
	float3 pos_2 = trianglesBuffer[id + 1].v;
	float3 pos_3 = trianglesBuffer[id + 2].v;
	
	float3 faceEdgeA = pos_2 - pos_1;
    float3 faceEdgeB = pos_3 - pos_1;
    float3 FaceNorm = normalize( cross(faceEdgeA, faceEdgeB) );

//    for( uint i=0; i<3; i++ )
//    {
        Out.posW = mul(float4(pos_1, 1), tWVP);
        Out.normW = FaceNorm;
        Out.texCd = float4(trianglesBuffer[id].v_info,1);
        OutputStream.Append( Out );
	
        Out.posW = mul(float4(pos_2, 1), tWVP);
        Out.normW = FaceNorm;
        Out.texCd = float4(trianglesBuffer[id+1].v_info,1);
        OutputStream.Append( Out );

        Out.posW = mul(float4(pos_3, 1), tWVP);
        Out.normW = FaceNorm;
        Out.texCd = float4(trianglesBuffer[id+2].v_info,1);
        OutputStream.Append( Out );

//    }
    OutputStream.RestartStrip();
}

// extrude triangle strip from vertices

float4 PS(GS_STREAMOUT In): SV_Target
{
    return float4(In.normW*0.5+0.5,1);
}

//==============================================================================
// TECHNIQUES ==================================================================
//==============================================================================

technique11 Simple_streamout
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_1() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
} 

technique11 face_normal_evaluation
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS_2() ) );
		SetPixelShader( CompileShader( ps_5_0, PS() ) );
	}
} 


