//@author: dottore
//@help: odd even sort
//@tags: GEA
//@credits: 

//==============================================================================
//PARAMETERS ===================================================================
//==============================================================================

// values to be sorted:
StructuredBuffer<float> ValuesBuffer;

RWStructuredBuffer<uint> IndecesBuffer : BACKBUFFER;

int iterId : ITERATIONINDEX ;
int count;// : ELEMENTCOUNT;// * 0.5;
//#DEFINE count elementCount*0.5;

// =============================================================================
// SORTING:

[numthreads(256, 1, 1)]
void CS_sort( uint3 DTid : SV_DispatchThreadID)
{
	bool pingpong = iterId%2;
	uint idOffsetRead = (int)pingpong * count;
	uint idOffsetWrite = (1-(int)pingpong) * count;

	// SELF INFO:
	uint selfID = IndecesBuffer[DTid.x+idOffsetRead];
	float selfValue = ValuesBuffer[selfID];

	int dir = (int(DTid.x%2)*2) - 1;
	dir = pingpong ? dir : -dir;
	
	int neighbCellId = DTid.x + dir;
	neighbCellId = min(max(neighbCellId,0), count-1);
	neighbCellId += idOffsetRead;
	
	uint neighbID = IndecesBuffer[neighbCellId];
	float neighbValue = ValuesBuffer[neighbID];
	
	if(dir>0)
	{
		IndecesBuffer[DTid.x+idOffsetWrite] = selfValue < neighbValue ? selfID : neighbID;	
	}
	else
	{
		IndecesBuffer[DTid.x+idOffsetWrite] = selfValue < neighbValue ? neighbID : selfID;			
	}
}
[numthreads(256, 1, 1)]
void CS_init( uint3 DTid : SV_DispatchThreadID)
{
	IndecesBuffer[DTid.x + (iterId%2)*count] = DTid.x ;
}

//==============================================================================
//==============================================================================
//==============================================================================

technique11 sort
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CS_sort() ) );
	}
}

technique11 Initialize
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CS_init() ) );
	}
}
