
int lineLength = 1; 
float segmDist = 0.1;
float damping = 0.95;

int iter ;
int elementCount;
int subIter : ITERATIONINDEX ;

float ciccio__mano;

struct data
{
	float3 pos;
	float3 vel;
};
RWStructuredBuffer<data> Output : BACKBUFFER;

StructuredBuffer<float3> emitterPos;
StructuredBuffer<float3> emitterVel;
StructuredBuffer<uint> indices;

[numthreads(1, 1, 1)]
void headsCS( uint3 DTid : SV_DispatchThreadID )
{
	uint binID = indices[DTid.x];
	uint id = binID * lineLength + iter*elementCount ;	
	
	Output[id].pos = emitterPos[binID];
	Output[id].vel = emitterVel[binID];
}

[numthreads(16, 1, 1)]
void tailsCS( uint3 DTid : SV_DispatchThreadID )
{
	int swap = (subIter%2) ? iter : !iter;
	uint iterOffset = (int)swap * elementCount;
	uint iterOffsetPrev = !swap * elementCount;
	
	uint groupID = DTid.x / (lineLength-1);
	uint binID = indices[groupID];
	uint segmID = DTid.x % (lineLength-1) + 1;
	
	uint id = (binID * lineLength) + segmID + iterOffset;
	uint oldId = (binID * lineLength) + segmID + iterOffsetPrev;
		
	float3 prevPos = Output[oldId].pos;
	float3 prevVel = Output[oldId].vel * damping;
	
	float3 trgPos = Output[oldId-1].pos;
	//float3 trgVel = Output[oldId-1].vel;
	
	float3 vec = (trgPos-prevPos);
	float dist = length(vec);
	
	float newVelAmount = damping * (dist-segmDist) / dist;
	float3 force = vec * newVelAmount + float3(0, -0.001, 0);
	
	//if(dist== 0) dist += 0.00001;
	//float3 force = (dist-segmDist) * normalize(vec) * 0.001;
	
	float3 newVel = prevVel + force;
	
	Output[id].vel = newVel;
	Output[id].pos = prevPos + force;//newVel;
}

[numthreads(256, 1, 1)]
void resetCS( uint3 DTid : SV_DispatchThreadID )
{
	float groupID = DTid.x / lineLength;
	float segmID = DTid.x % lineLength;
				
	Output[DTid.x].pos = float3(segmID*0.1, groupID*0.1, 0);
	Output[DTid.x].vel = 0.0001;
}

//==============================================================================
//==============================================================================
//==============================================================================

technique11 setHeads
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, headsCS() ) );
	}
}

technique11 setTails
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, tailsCS() ) );
	}
}

technique11 resetBin
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, resetCS() ) );
	}
}
